import json

from django.contrib import admin
from django.template.defaultfilters import linebreaksbr
from django.utils.html import format_html
from django.utils.translation import gettext as _

from .models import Attachment, Email, Log


class LogInline(admin.StackedInline):
    model = Log
    readonly_fields = [
        "type",
        "timestamp",
        "esp",
        "event_id",
        "reject_reason",
        "mta_response",
        "tags",
        "user_agent",
        "click_url",
        "raw",
    ]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "type",
                    "timestamp",
                    "tags",
                ),
            },
        ),
        (
            _("Details"),
            {
                "fields": (
                    "esp",
                    "event_id",
                    "mta_response",
                    "reject_reason",
                    "user_agent",
                    "click_url",
                    "raw",
                ),
                "classes": ["collapse"],
            },
        ),
    )
    extra = 0

    def has_add_permission(self, request, obj):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class AttachmentInline(admin.StackedInline):
    model = Attachment
    verbose_name = "Attachment"
    verbose_name_plural = "Attachments"
    can_delete = False
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "file",
                    "name",
                    "mimetype",
                )
            },
        ),
    )
    readonly_fields = (
        "file",
        "name",
        "mimetype",
    )
    extra = 0

    def has_add_permission(self, request, obj):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class EmailAdmin(admin.ModelAdmin):
    list_display = ["recipients", "from_email", "subject", "date_sent", "ok"]
    list_filter = ["date_sent", "ok"]
    readonly_fields = [
        "from_email",
        "recipients",
        "cc_recipients",
        "bcc_recipients",
        "reply_to",
        "headers",
        "subject",
        "body_formatted",
        "html_message",
        "html_message_preview",
        "date_sent",
        "ok",
    ]
    inlines = [
        AttachmentInline,
        LogInline,
    ]
    search_fields = [
        "subject",
        "body",
        "recipients",
        "cc_recipients",
        "bcc_recipients",
        "extra_headers",
    ]
    exclude = ["body", "html_message", "extra_headers"]
    fieldsets = [
        (
            None,
            {
                "fields": (
                    "from_email",
                    "recipients",
                    "cc_recipients",
                    "bcc_recipients",
                    "reply_to",
                    "headers",
                    "subject",
                    "body_formatted",
                    "html_message_preview",
                    "date_sent",
                    "ok",
                )
            },
        ),
        (
            "Plain HTML message",
            {
                "classes": ("collapse",),
                "fields": ("html_message",),
            },
        ),
    ]

    def has_delete_permission(self, request, *args, **kwargs):
        return request.user.is_superuser

    def has_add_permission(self, *args, **kwargs):
        return False

    def headers(self, obj) -> str:
        return format_html("<pre>{}</pre>", json.dumps(obj.extra_headers, indent=2))

    headers.short_description = "Extra headers"

    def html_message_preview(self, obj):
        if obj.html_message:
            return format_html(
                '<iframe style="border: 1px solid #e8e8e8; max-width: 800px; max-height: 600px" srcdoc="{}"></iframe>',  # noqa: ignore E501
                obj.html_message,
            )
        else:
            return "No HTML content"

    html_message_preview.short_description = "HTML message"

    def body_formatted(self, obj):
        return linebreaksbr(obj.body)

    body_formatted.short_description = "body"


admin.site.register(Email, EmailAdmin)



# Generated by Django 4.1.7 on 2023-03-09 05:31

from django.db import migrations, models
import email_log.models


class Migration(migrations.Migration):

    dependencies = [
        ("email_log", "0003_email_html_message_attachment"),
    ]

    operations = [
        migrations.AlterField(
            model_name="attachment",
            name="file",
            field=models.FileField(
                upload_to=email_log.models.get_attachment_path, verbose_name="file"
            ),
        ),
    ]



import pathlib

from django.contrib.postgres.indexes import GinIndex
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.translation import gettext_lazy as _

from .conf import settings


class Email(models.Model):
    """Model to store outgoing email information"""

    from_email = models.TextField(_("from email"))
    recipients = models.TextField(_("recipients"))
    cc_recipients = models.TextField(_("cc recipients"), blank=True)
    bcc_recipients = models.TextField(_("bcc recipients"), blank=True)
    reply_to = models.TextField(_("reply to"), blank=True, default="")
    extra_headers = models.JSONField(encoder=DjangoJSONEncoder, default=dict)
    subject = models.TextField(_("subject"))
    body = models.TextField(_("body"))
    ok = models.BooleanField(_("ok"), default=False, db_index=True)
    date_sent = models.DateTimeField(_("date sent"), auto_now_add=True, db_index=True)
    html_message = models.TextField(_("HTML message"), blank=True)

    def __str__(self):
        return "{s.recipients}: {s.subject}".format(s=self)

    class Meta:
        verbose_name = _("email")
        verbose_name_plural = _("emails")
        ordering = ("-date_sent",)
        indexes = [
            GinIndex(fields=["extra_headers"]),
        ]


def get_attachment_path(instance, filename: str) -> str:
    """Return attachments path from settings

    If attachments path is callable then call it and return result. Otherwise
    return path concatenated with the filename.

    """
    path = settings.EMAIL_LOG_ATTACHMENTS_PATH
    if callable(path):
        return path(instance, filename)
    return str(pathlib.Path(str(path)) / filename)


class Attachment(models.Model):
    """Model to store attachments of outgoing email"""

    file = models.FileField(
        _("file"),
        upload_to=get_attachment_path,
    )
    name = models.CharField(_("name"), max_length=255, help_text=_("filename"))
    email = models.ForeignKey(
        Email,
        related_name="attachments",
        verbose_name=_("email"),
        on_delete=models.CASCADE,
    )
    mimetype = models.CharField(max_length=255, default="", blank=True)

    class Meta:
        verbose_name = _("attachment")
        verbose_name_plural = _("attachments")

    def __str__(self):
        return self.name


class Log(models.Model):
    email = models.ForeignKey(
        Email, related_name="logs", verbose_name=_("email"), on_delete=models.CASCADE
    )
    esp = models.CharField(verbose_name=_("ESP"), max_length=255)
    metadata = models.JSONField()
    type = models.CharField(max_length=255)
    timestamp = models.DateTimeField()
    event_id = models.TextField()
    mta_response = models.TextField(verbose_name=_("MTA Response"), null=True)
    reject_reason = models.TextField(null=True)
    tags = models.JSONField()
    user_agent = models.TextField(null=True)
    click_url = models.URLField(verbose_name="Click URL", null=True, max_length=4192)
    raw = models.JSONField()

    class Meta:
        ordering = ["email", "timestamp"]

    def __str__(self):
        return self.type



# Generated by Django 5.1 on 2025-07-16 19:19

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("email_log", "0005_email_bcc_recipients_email_cc_recipients"),
    ]

    operations = [
        migrations.AddField(
            model_name="email",
            name="extra_headers",
            field=models.JSONField(
                default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder
            ),
        ),
        migrations.AddField(
            model_name="email",
            name="reply_to",
            field=models.TextField(blank=True, default="", verbose_name="reply to"),
        ),
    ]



# encoding: utf8
from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Email",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("from_email", models.TextField(verbose_name="from e-mail")),
                ("recipients", models.TextField(verbose_name="recipients")),
                ("subject", models.TextField(verbose_name="subject")),
                ("body", models.TextField(verbose_name="body")),
                (
                    "ok",
                    models.BooleanField(
                        default=False, db_index=True, verbose_name="ok"
                    ),
                ),
                (
                    "date_sent",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="date sent", db_index=True
                    ),
                ),
            ],
            options={
                "ordering": ("-date_sent",),
                "verbose_name": "e-mail",
                "verbose_name_plural": "e-mails",
            },
            bases=(models.Model,),
        ),
    ]



from django.apps import AppConfig
from django.conf import settings as django_settings

from .conf import settings


class EmailLogConfig(AppConfig):
    name = "email_log"
    verbose_name = "Email log"
    default_auto_field = "django.db.models.AutoField"

    def ready(self):
        if "anymail" in django_settings.INSTALLED_APPS:
            from anymail.signals import post_send, tracking
            from .signal_handlers import handle_tracking_event, log_successful_email

            if settings.EMAIL_LOG_CONNECT_ANYMAIL_SIGNALS:
                tracking.connect(handle_tracking_event)
                post_send.connect(log_successful_email)



# Generated by Django 5.1 on 2025-08-01 20:27

import django.contrib.postgres.indexes
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("email_log", "0006_email_extra_headers_email_reply_to"),
    ]

    operations = [
        migrations.CreateModel(
            name="Log",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "email",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="logs",
                        to="email_log.email",
                        verbose_name="email",
                    ),
                ),
                ("esp", models.CharField(verbose_name="ESP", max_length=255)),
                ("metadata", models.JSONField()),
                ("type", models.CharField(max_length=255)),
                ("timestamp", models.DateTimeField()),
                ("event_id", models.TextField()),
                (
                    "mta_response",
                    models.TextField(null=True, verbose_name="MTA Response"),
                ),
                ("reject_reason", models.TextField(null=True)),
                ("tags", models.JSONField()),
                ("user_agent", models.TextField(null=True)),
                (
                    "click_url",
                    models.URLField(
                        null=True, verbose_name="Click URL", max_length=4192
                    ),
                ),
                ("raw", models.JSONField()),
            ],
            options={
                "ordering": ["email", "timestamp"],
            },
        ),
        migrations.AddIndex(
            model_name="email",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["extra_headers"], name="email_log_e_extra_h_b45426_gin"
            ),
        ),
    ]



# Generated by Django 4.2.13 on 2024-10-04 06:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("email_log", "0004_alter_attachment_file"),
    ]

    operations = [
        migrations.AddField(
            model_name="email",
            name="bcc_recipients",
            field=models.TextField(blank=True, verbose_name="bcc recipients"),
        ),
        migrations.AddField(
            model_name="email",
            name="cc_recipients",
            field=models.TextField(blank=True, verbose_name="cc recipients"),
        ),
    ]



from typing import TYPE_CHECKING, Any, Type

from django.conf import settings
from django.core.mail import EmailMessage
from django.core.mail.backends.base import BaseEmailBackend
from django.db import transaction

from email_log.backends import EmailBackend as EmailLogBackend
from email_log.models import Email, Log as EmailLog

if TYPE_CHECKING:  # pragma: no cover
    from anymail.webhooks.base import AnymailCoreWebhookView
    from anymail.message import AnymailStatus
    from anymail.signals import AnymailTrackingEvent


def handle_tracking_event(
    sender: "Type[AnymailCoreWebhookView]",
    event: "AnymailTrackingEvent",
    esp_name: str,
    **kwargs: Any,
) -> None:
    if (
        email := Email.objects.filter(extra_headers__anymail_id=event.message_id)
        .only("pk")
        .first()
    ):
        EmailLog.objects.create(
            email=email,
            type=event.event_type,
            esp=esp_name,
            metadata=event.metadata,
            timestamp=event.timestamp,
            event_id=event.event_id,
            reject_reason=event.reject_reason,
            mta_response=event.mta_response,
            tags=event.tags,
            user_agent=event.user_agent,
            click_url=event.click_url,
            raw=event.esp_event,
        )


def log_successful_email(
    sender: "BaseEmailBackend",
    message: "EmailMessage",
    status: "AnymailStatus",
    esp_name: str,
    **kwargs: Any,
) -> None:
    backend = EmailLogBackend()
    message_headers = dict(message.message()._headers)
    for key in list(message_headers.keys()):
        if key.lower() in [
            "content-type",
            "date",
            "from",
            "mime-version",
            "subject",
            "to",
        ]:
            del message_headers[key]
    with transaction.atomic():
        email = Email.objects.create(
            recipients="; ".join(message.to),
            cc_recipients="; ".join(message.cc) if message.cc else "",
            bcc_recipients="; ".join(message.bcc) if message.bcc else "",
            reply_to="; ".join(message.reply_to) if message.reply_to else "",
            extra_headers=(
                message_headers
                | message.extra_headers  # noqa: ignore W503
                | {"anymail_id": status.message_id}  # noqa: ignore W503
            ),
            subject=message.subject,
            body=message.body,
            html_message=backend._get_html_message(message),
            ok=True,
        )

        if getattr(settings, "EMAIL_LOG_SAVE_ATTACHMENTS", True):
            backend._log_attachments(email=email, message=message)



from django.core.files.base import ContentFile
from django.core.mail import EmailMessage
from django.core.mail import get_connection
from django.core.mail.backends.base import BaseEmailBackend
from email.mime.base import MIMEBase

import logging
from .conf import settings
from .models import Attachment, Email


class EmailBackend(BaseEmailBackend):
    """Wrapper email backend that records all emails in a database model"""

    def __init__(self, **kwargs):
        super(EmailBackend, self).__init__(**kwargs)
        self.connection = get_connection(settings.EMAIL_LOG_BACKEND, **kwargs)

    def send_messages(self, email_messages):
        num_sent = 0
        for message in email_messages:
            recipients = "; ".join(message.to)
            cc_recipients = "; ".join(message.cc) if message.cc else ""
            bcc_recipients = "; ".join(message.bcc) if message.bcc else ""
            reply_to = "; ".join(message.reply_to) if message.reply_to else ""
            email = None
            html_message = self._get_html_message(message)
            try:
                email = Email.objects.create(
                    from_email=message.from_email,
                    recipients=recipients,
                    cc_recipients=cc_recipients,
                    bcc_recipients=bcc_recipients,
                    reply_to=reply_to,
                    extra_headers=message.extra_headers,
                    subject=message.subject,
                    body=message.body,
                    html_message=html_message,
                )
            except Exception:
                logging.error(
                    "Failed to save email to database (create)", exc_info=True
                )

            if settings.EMAIL_LOG_SAVE_ATTACHMENTS:
                self._log_attachments(email, message)

            message.connection = self.connection
            num_sent += message.send()
            if num_sent > 0 and email:
                email.ok = True
                try:
                    email.save()
                except Exception:
                    logging.error(
                        "Failed to save email to database (update)", exc_info=True
                    )
        return num_sent

    def _get_html_message(self, message: EmailMessage) -> str:
        """Retrieve html message from the email message."""
        if hasattr(message, "alternatives") and len(message.alternatives) > 0:
            for alternative in message.alternatives:
                if alternative[1] == "text/html":
                    return alternative[0]
        return ""

    def _log_attachments(self, email: Email, message: EmailMessage):
        """Retrieve all attachments from email and create them."""
        attachment_files = {}
        for attachment in message.attachments:
            if isinstance(attachment, MIMEBase):
                attachment_files[attachment.get_filename()] = {
                    "file": ContentFile(attachment.get_payload()),
                }
            else:
                name, content, type = attachment
                attachment_files[name] = {
                    "file": ContentFile(content),
                }
                if type != "application/octet-stream":
                    attachment_files[name].update({"mimetype": type})

        if attachment_files:
            self._create_attachments(email, attachment_files)

    def _create_attachments(self, email: Email, files: dict):
        """Create attachments and save it to database."""
        for filename, filedata in files.items():
            content = filedata.get("file", None)
            mimetype = filedata.get("mimetype", None)
            attachment = Attachment()

            attachment.name = filename
            attachment.email = email
            if mimetype:
                attachment.mimetype = mimetype

            attachment.file.save(
                filename,
                content=content,
                save=True,
            )
